from .client_abstract import ClientAbstract
from .client_abstract import TagAbstract
from .credentials import CredentialsInterface
from .credentials import Anonymous
from .credentials import ApiKey
from .credentials import HttpBasic
from .credentials import HttpBearer
from .credentials import OAuth2
from .exceptions import ClientException
from .exceptions import KnownStatusCodeException
from .exceptions import UnknownStatusCodeException
from .exceptions import ParserException
from .exceptions import AccessTokenRequestException
from .exceptions import FoundNoAccessTokenException
from .exceptions import InvalidAccessTokenException
from .exceptions import InvalidCredentialsException
from .parser import Parser
from .token_store import TokenStoreInterface
from .token_store import MemoryTokenStore



from typing import Optional

from .token_store import TokenStoreInterface


class CredentialsInterface:
    pass


class Anonymous(CredentialsInterface):
    pass


class ApiKey(CredentialsInterface):
    token: str = None
    name: str = None
    in_: str = None

    def __init__(self, token: str, name: str, in_: str):
        self.token = token
        self.name = name
        self.in_ = in_


class HttpBasic(CredentialsInterface):
    username: str = None
    password: str = None

    def __init__(self, username: str, password: str):
        self.username = username
        self.password = password


class HttpBearer(CredentialsInterface):
    def __init__(self, token: str):
        self.token = token


class OAuth2(CredentialsInterface):
    client_id: str = None
    client_secret: str = None
    token_url: str = None
    authorization_url: str = None
    token_store: Optional[TokenStoreInterface] = None
    scopes: Optional[list[str]] = None

    def __init__(self, client_id: str, client_secret: str, token_url: str, authorization_url: str,
                 token_store: Optional[TokenStoreInterface], scopes: Optional[list[str]]):
        self.client_id = client_id
        self.client_secret = client_secret
        self.token_url = token_url
        self.authorization_url = authorization_url
        self.token_store = token_store
        self.scopes = scopes



from requests import Session

from .authenticator import AuthenticatorFactory, HttpClientFactory, AuthenticatorInterface
from .credentials import CredentialsInterface
from .parser import Parser


class ClientAbstract:
    USER_AGENT = "SDKgen Client v2.0"

    authenticator: AuthenticatorInterface = None
    http_client: Session = None
    parser: Parser = None

    def __init__(self, base_url: str, credentials: CredentialsInterface):
        self.authenticator = AuthenticatorFactory.factory(credentials)
        self.http_client = HttpClientFactory(self.authenticator).factory()
        self.parser = Parser(base_url)


class TagAbstract:
    http_client: Session = None
    parser: Parser = None

    def __init__(self, http_client: Session, parser: Parser):
        self.http_client = http_client
        self.parser = parser



import time
import unittest

from src.sdkgen.access_token import AccessToken
from tests.generated.test_request import TestRequest


class TestSerialize(unittest.TestCase):
    def test_serialize(self):
        json = '{"int": 1337, "float": 13.37, "string": "foobar", "bool": true, "arrayScalar": ["foo", "bar"], "arrayObject": [{"id": 1, "name": "foo"}, {"id": 1, "name": "bar"}], "mapScalar": {"bar": "foo", "foo": "bar"}, "mapObject": {"bar": {"id": 1, "name": "bar"}, "foo": {"id": 1, "name": "foo"}}, "object": {"id": 1, "name": "foo"}}'

        response = TestRequest.model_validate_json(json_data=json)

        self.assertEqual(1337, response.int_)
        self.assertEqual(13.37, response.float_)
        self.assertEqual("foobar", response.string)
        self.assertEqual(True, response.bool_)
        self.assertEqual(["foo", "bar"], response.array_scalar)

    def test_serialize_access_token(self):
        json = '{"access_token": "foobar", "expires_in": 1337}'

        token = AccessToken.model_validate_json(json_data=json)
        now = time.time()

        self.assertEqual("foobar", token.access_token)
        self.assertEqual(1337, token.expires_in)
        self.assertTrue(token.get_expires_in_timestamp() >= now + 1337)


if __name__ == '__main__':
    unittest.main()



import base64
import time
import urllib.parse
from typing import List, Optional

import requests
from requests import Session, Response
from requests.auth import AuthBase

from .access_token import AccessToken
from .credentials import HttpBasic, HttpBearer, ApiKey, OAuth2, Anonymous, CredentialsInterface
from .exceptions import InvalidAccessTokenException, InvalidCredentialsException
from .token_store import MemoryTokenStore, TokenStoreInterface


class AuthenticatorInterface(AuthBase):
    pass


class AnonymousAuthenticator(AuthenticatorInterface):
    def __init__(self, credentials: Anonymous):
        self.credentials = credentials

    def __call__(self, request):
        return request


class HttpBasicAuthenticator(AuthenticatorInterface):
    credentials: HttpBasic = None

    def __init__(self, credentials: HttpBasic):
        self.credentials = credentials

    def __call__(self, request):
        basic = base64.b64encode((self.credentials.username + ":" + self.credentials.password).encode('utf-8')).decode('ascii')
        request.headers["Authorization"] = "Basic " + basic
        return request


class HttpBearerAuthenticator(AuthenticatorInterface):
    credentials: HttpBearer = None

    def __init__(self, credentials: HttpBearer):
        self.credentials = credentials

    def __call__(self, request):
        request.headers["Authorization"] = "Bearer " + self.credentials.token
        return request


class ApiKeyAuthenticator(AuthenticatorInterface):
    credentials: ApiKey = None

    def __init__(self, credentials: ApiKey):
        self.credentials = credentials

    def __call__(self, request):
        request.headers[self.credentials.name] = self.credentials.token
        return request


class OAuth2Authenticator(AuthenticatorInterface):
    EXPIRE_THRESHOLD: int = 60 * 10

    credentials: OAuth2 = None
    scopes: Optional[list[str]] = None
    token_store: TokenStoreInterface = None

    def __init__(self, credentials: OAuth2):
        self.credentials = credentials
        self.scopes = credentials.scopes
        if credentials.token_store:
            self.token_store = credentials.token_store
        else:
            self.token_store = MemoryTokenStore()

    def __call__(self, request):
        request.headers["Authorization"] = "Bearer " + self.get_access_token()
        return request

    def build_redirect_url(self, redirect_url: str, scopes: List[str], state: str) -> str:
        parameters = {
            "response_type": "code",
            "client_id": self.credentials.client_id,
        }

        if redirect_url:
            parameters["redirect_uri"] = redirect_url

        if scopes:
            parameters["scope"] = ",".join(scopes)
        elif self.scopes:
            parameters["scope"] = ",".join(self.scopes)

        if state:
            parameters["state"] = state

        return self.credentials.authorization_url + "?" + urllib.parse.urlencode(parameters)

    def fetch_access_token_by_code(self, code: str) -> AccessToken:
        credentials = HttpBasic(self.credentials.client_id, self.credentials.client_secret)

        headers = {
            "Accept": "application/json",
        }

        data = {
            "grant_type": "authorization_code",
            "code": code,
        }

        response = self.new_http_client(credentials).post(self.credentials.token_url, headers=headers, data=data)

        return self.parse_token_response(response)

    def fetch_access_token_by_client_credentials(self) -> AccessToken:
        credentials = HttpBasic(self.credentials.client_id, self.credentials.client_secret)

        headers = {
            "Accept": "application/json",
        }

        data = {
            "grant_type": "client_credentials",
        }

        if self.scopes:
            data["scope"] = ",".join(self.scopes)

        response = self.new_http_client(credentials).post(self.credentials.token_url, headers=headers, data=data)

        return self.parse_token_response(response)

    def fetch_access_token_by_refresh(self, refresh_token: str) -> AccessToken:
        credentials = HttpBearer(self.get_access_token(False, 0))

        headers = {
            "Accept": "application/json",
        }

        data = {
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
        }

        response = self.new_http_client(credentials).post(self.credentials.token_url, headers=headers, data=data)

        return self.parse_token_response(response)

    def get_access_token(self, automatic_refresh: bool = True, expire_threshold: int = EXPIRE_THRESHOLD) -> str:
        timestamp = time.time()

        access_token = self.credentials.token_store.get()
        if not access_token or access_token.get_expires_in_timestamp() < timestamp:
            access_token = self.fetch_access_token_by_client_credentials()

        if access_token.get_expires_in_timestamp() > (timestamp + expire_threshold):
            return access_token.access_token

        if automatic_refresh and access_token.refresh_token:
            access_token = self.fetch_access_token_by_refresh(access_token.refresh_token)

        return access_token.access_token

    def parse_token_response(self, response: Response) -> AccessToken:
        if response.status_code != 200:
            raise InvalidAccessTokenException(
                "Could not obtain access token, received a non successful status code: " + str(response.status_code))

        token = AccessToken.model_validate_json(json_data=response.content)

        self.token_store.persist(token)

        return token

    def new_http_client(self, credentials: CredentialsInterface) -> Session:
        return HttpClientFactory(AuthenticatorFactory.factory(credentials)).factory()


class AuthenticatorFactory:
    @staticmethod
    def factory(credentials: CredentialsInterface) -> AuthenticatorInterface:
        if isinstance(credentials, HttpBasic):
            return HttpBasicAuthenticator(credentials)
        elif isinstance(credentials, HttpBearer):
            return HttpBearerAuthenticator(credentials)
        elif isinstance(credentials, ApiKey):
            return ApiKeyAuthenticator(credentials)
        elif isinstance(credentials, OAuth2):
            return OAuth2Authenticator(credentials)
        elif isinstance(credentials, Anonymous):
            return AnonymousAuthenticator(credentials)
        else:
            raise InvalidCredentialsException("Could not find authenticator for credentials")


class HttpClientFactory:
    authenticator: AuthenticatorInterface = None

    def __init__(self, authenticator: AuthenticatorInterface):
        self.authenticator = authenticator

    def factory(self) -> Session:
        session = requests.Session()
        session.auth = self.authenticator
        session.headers['User-Agent'] = 'SDKgen Client v2.0'
        session.headers['Accept'] = 'application/json'
        return session



import string
import datetime


class Parser:
    base_url: str = None

    def __init__(self, base_url: str):
        self.base_url = self.normalize_url(base_url)

    def url(self, path: str, parameters: dict[str, any]) -> str:
        return self.base_url + "/" + self.substitute_parameters(path, parameters)

    @classmethod
    def substitute_parameters(cls, path: str, parameters: dict[str, any]) -> str:
        parts = path.split("/")
        result = []

        for part in parts:
            if part is None or part == "":
                continue

            name = ""
            if part.startswith(":"):
                name = part[1:]
            elif part.startswith("$"):
                try:
                    pos = part.index("<")
                    name = part[1:pos]
                except ValueError:
                    name = part[1:]
            elif part.startswith("{") and part.endswith("}"):
                name = part[1:len(part) - 1]

            if name in parameters:
                part = cls.to_string(parameters[name])

            result.append(part)

        return "/".join(result)

    @classmethod
    def query(cls, parameters: dict[str, any], struct_names: list[str] = None) -> dict[str, any]:
        result: dict[str, any] = {}
        for name, value in parameters.items():
            if value is None:
                continue

            if struct_names and name in struct_names:
                result = result | cls.query(value.model_dump())
            else:
                result[name] = cls.to_string(value)

        return result

    @classmethod
    def to_string(cls, value: any) -> string:
        t = type(value)
        if t is int:
            return str(value)
        elif t is float:
            return str(value)
        elif t is bool:
            return "1" if value else "0"
        elif t is str:
            return str(value)
        elif t is datetime.date:
            return value.isoformat()
        elif t is datetime.datetime:
            return value.isoformat() + "Z"
        elif t is datetime.time:
            return value.isoformat()
        else:
            return ""

    @classmethod
    def normalize_url(cls, value: string) -> string:
        if value.endswith("/"):
            value = value[0:len(value) - 1]
        return value



import unittest
from .generated.client import Client
from .generated.test_request import TestRequest
from .generated.test_object import TestObject


class TestIntegration(unittest.TestCase):
    def test_client_get_all(self):
        client = Client.build('my_token')

        response = client.product().get_all(8, 16, 'foobar')

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'GET')
        self.assertEqual(response.args['startIndex'], '8')
        self.assertEqual(response.args['count'], '16')
        self.assertEqual(response.args['search'], 'foobar')
        self.assertEqual(response.json, None)

    def test_client_create(self):
        client = Client.build('my_token')

        payload = self.new_payload()
        response = client.product().create(payload)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'POST')
        self.assertEqual(response.json.model_dump_json(by_alias=True), '{"int":1337,"float":13.37,"string":"foobar","bool":true,"dateString":"2024-09-22","dateTimeString":"2024-09-22T10:09:00","timeString":"10:09:00","arrayScalar":["foo","bar"],"arrayObject":[{"id":1,"name":"foo"},{"id":1,"name":"bar"}],"mapScalar":{"bar":"foo","foo":"bar"},"mapObject":{"bar":{"id":1,"name":"bar"},"foo":{"id":1,"name":"foo"}},"object":{"id":1,"name":"foo"}}')

    def test_client_update(self):
        client = Client.build('my_token')

        payload = self.new_payload()
        response = client.product().update(1, payload)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'PUT')
        self.assertEqual(response.json.model_dump_json(by_alias=True), '{"int":1337,"float":13.37,"string":"foobar","bool":true,"dateString":"2024-09-22","dateTimeString":"2024-09-22T10:09:00","timeString":"10:09:00","arrayScalar":["foo","bar"],"arrayObject":[{"id":1,"name":"foo"},{"id":1,"name":"bar"}],"mapScalar":{"bar":"foo","foo":"bar"},"mapObject":{"bar":{"id":1,"name":"bar"},"foo":{"id":1,"name":"foo"}},"object":{"id":1,"name":"foo"}}')

    def test_client_patch(self):
        client = Client.build('my_token')

        payload = self.new_payload()
        response = client.product().patch(1, payload)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'PATCH')
        self.assertEqual(response.json.model_dump_json(by_alias=True), '{"int":1337,"float":13.37,"string":"foobar","bool":true,"dateString":"2024-09-22","dateTimeString":"2024-09-22T10:09:00","timeString":"10:09:00","arrayScalar":["foo","bar"],"arrayObject":[{"id":1,"name":"foo"},{"id":1,"name":"bar"}],"mapScalar":{"bar":"foo","foo":"bar"},"mapObject":{"bar":{"id":1,"name":"bar"},"foo":{"id":1,"name":"foo"}},"object":{"id":1,"name":"foo"}}')

    def test_client_delete(self):
        client = Client.build('my_token')

        response = client.product().delete(1)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'DELETE')
        self.assertEqual(response.json, None)

    def test_client_binary(self):
        client = Client.build('my_token')

        payload = bytearray([0x66, 0x6F, 0x6F, 0x62, 0x61, 0x72])

        response = client.product().binary(payload)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'POST')
        self.assertEqual(response.data, 'foobar')

    def test_client_form(self):
        client = Client.build('my_token')

        payload = {'foo': 'bar'}

        response = client.product().form(payload)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'POST')
        self.assertEqual(response.form['foo'], 'bar')

    def test_client_json(self):
        client = Client.build('my_token')

        payload = {'string': 'bar'}

        response = client.product().json(payload)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'POST')
        self.assertEqual(response.json.string, 'bar')

    def test_client_multipart(self):
        client = Client.build('my_token')

        payload = {'foo': bytearray([0x66, 0x6F, 0x6F, 0x62, 0x61, 0x72])}

        response = client.product().multipart(payload)

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'POST')
        self.assertEqual(response.files['foo'], 'foobar')

    def test_client_text(self):
        client = Client.build('my_token')

        response = client.product().text('foobar')

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'POST')
        self.assertEqual(response.data, 'foobar')

    def test_client_xml(self):
        client = Client.build('my_token')

        response = client.product().xml('<foo>bar</foo>')

        self.assertEqual(response.headers['Authorization'], 'Bearer my_token')
        self.assertEqual(response.headers['Accept'], 'application/json')
        self.assertEqual(response.headers['User-Agent'], 'SDKgen Client v2.0')
        self.assertEqual(response.method, 'POST')
        self.assertEqual(response.data, '<foo>bar</foo>')

    def new_payload(self) -> TestRequest:
        object_foo = TestObject()
        object_foo.id = 1
        object_foo.name = 'foo'
        object_bar = TestObject()
        object_bar.id = 1
        object_bar.name = 'bar'

        array_scalar = ['foo', 'bar']
        array_object = [object_foo, object_bar]

        map_scalar = {'foo': 'bar', 'bar': 'foo'}
        map_object = {'foo': object_foo, 'bar': object_bar}

        request = TestRequest()
        request.int_ = 1337
        request.float_ = 13.37
        request.string = 'foobar'
        request.bool_ = True
        request.date_string = '2024-09-22'
        request.date_time_string = '2024-09-22T10:09:00'
        request.time_string = '10:09:00'
        request.array_scalar = array_scalar
        request.array_object = array_object
        request.map_scalar = map_scalar
        request.map_object = map_object
        request.object = object_foo
        return request


if __name__ == '__main__':
    unittest.main()



import unittest

from src.sdkgen import MemoryTokenStore
from src.sdkgen.access_token import AccessToken


class TestTokenStore(unittest.TestCase):
    def test_memory_token_store(self):
        token = AccessToken()
        token.access_token = 'foobar'

        store = MemoryTokenStore()

        store.persist(token)

        stored_token = store.get()

        self.assertEqual(token.access_token, stored_token.access_token)

        store.remove()

        self.assertEqual(None, store.get())


if __name__ == '__main__':
    unittest.main()



import datetime
import unittest

from src.sdkgen import Parser
from tests.generated.test_object import TestObject


class TestParser(unittest.TestCase):
    def test_url(self):
        parser = Parser("https://api.acme.com/")

        self.assertEqual(parser.url("/foo/bar", {}), "https://api.acme.com/foo/bar")
        self.assertEqual(parser.url("/foo/:bar", {"bar": "foo"}), "https://api.acme.com/foo/foo")
        self.assertEqual(parser.url("/foo/$bar<[0-9]+>", {"bar": "foo"}), "https://api.acme.com/foo/foo")
        self.assertEqual(parser.url("/foo/$bar", {"bar": "foo"}), "https://api.acme.com/foo/foo")
        self.assertEqual(parser.url("/foo/{bar}", {"bar": "foo"}), "https://api.acme.com/foo/foo")
        self.assertEqual(parser.url("/foo/:bar/bar", {"bar": "foo"}), "https://api.acme.com/foo/foo/bar")
        self.assertEqual(parser.url("/foo/$bar<[0-9]+>/bar", {"bar": "foo"}), "https://api.acme.com/foo/foo/bar")
        self.assertEqual(parser.url("/foo/$bar/bar", {"bar": "foo"}), "https://api.acme.com/foo/foo/bar")
        self.assertEqual(parser.url("/foo/{bar}/bar", {"bar": "foo"}), "https://api.acme.com/foo/foo/bar")

        self.assertEqual(parser.url("/foo/:bar", {"bar": None}), "https://api.acme.com/foo/")
        self.assertEqual(parser.url("/foo/:bar", {"bar": 1337}), "https://api.acme.com/foo/1337")
        self.assertEqual(parser.url("/foo/:bar", {"bar": 13.37}), "https://api.acme.com/foo/13.37")
        self.assertEqual(parser.url("/foo/:bar", {"bar": True}), "https://api.acme.com/foo/1")
        self.assertEqual(parser.url("/foo/:bar", {"bar": False}), "https://api.acme.com/foo/0")
        self.assertEqual(parser.url("/foo/:bar", {"bar": "foo"}), "https://api.acme.com/foo/foo")

    def test_query(self):
        parser = Parser("https://api.acme.com/")

        test = TestObject()
        test.name = "foo"

        parameters = {
            "null": None,
            "int": 1337,
            "float": 13.37,
            "true": True,
            "false": False,
            "string": "foo",
            "date": datetime.date(2023, 2, 21),
            "datetime": datetime.datetime(2023, 2, 21, 19, 19, 0),
            "time": datetime.time(19, 19, 0),
            "args": test,
        }

        result = parser.query(parameters, ["args"])

        self.assertEqual(result["int"], "1337")
        self.assertEqual(result["float"], "13.37")
        self.assertEqual(result["true"], "1")
        self.assertEqual(result["false"], "0")
        self.assertEqual(result["string"], "foo")
        self.assertEqual(result["date"], "2023-02-21")
        self.assertEqual(result["datetime"], "2023-02-21T19:19:00Z")
        self.assertEqual(result["time"], "19:19:00")
        self.assertEqual(result["name"], "foo")


if __name__ == '__main__':
    unittest.main()



class ClientException(Exception):
    pass


class KnownStatusCodeException(ClientException):
    pass


class UnknownStatusCodeException(ClientException):
    pass


class ParserException(ClientException):
    pass


class AccessTokenRequestException(ClientException):
    pass


class FoundNoAccessTokenException(ClientException):
    pass


class InvalidAccessTokenException(ClientException):
    pass


class InvalidCredentialsException(ClientException):
    pass



from typing import Optional

from .access_token import AccessToken


class TokenStoreInterface:
    def get(self) -> AccessToken:
        pass

    def persist(self, token: AccessToken):
        pass

    def remove(self):
        pass


class MemoryTokenStore(TokenStoreInterface):
    token: Optional[AccessToken] = None

    def __init__(self):
        self.token = None

    def get(self) -> AccessToken:
        return self.token

    def persist(self, token: AccessToken):
        self.token = token

    def remove(self):
        self.token = None



import time
from typing import Optional

from pydantic import BaseModel, Field


class AccessToken(BaseModel):
    access_token: Optional[str] = Field(default=None, alias="access_token")
    token_type: Optional[str] = Field(default=None, alias="token_type")
    expires_in: Optional[int] = Field(default=None, alias="expires_in")
    refresh_token: Optional[str] = Field(default=None, alias="refresh_token")
    scope: Optional[str] = Field(default=None, alias="scope")

    def get_expires_in_timestamp(self):
        now = time.time()

        expires_in = self.expires_in
        if self.expires_in < 529196400:
            expires_in = now + expires_in

        return expires_in
